use godot::prelude::*;
use crate::face_data_object::FaceDataObject;

#[derive(GodotConvert, Var, Export, Debug)]
#[godot(via = GString)]
pub enum ReadBlendShape {
    EyeBlinkLeft,
    EyeLookDownLeft,
    EyeLookInLeft,
    EyeLookOutLeft,
    EyeLookUpLeft,
    EyeSquintLeft,
    EyeWideLeft,
    EyeBlinkRight,
    EyeLookDownRight,
    EyeLookInRight,
    EyeLookOutRight,
    EyeLookUpRight,
    EyeSquintRight,
    EyeWideRight,
    JawForward,
    JawLeft,
    JawRight,
    JawOpen,
    MouthClose,
    MouthFunnel,
    MouthPucker,
    MouthLeft,
    MouthRight,
    MouthSmileLeft,
    MouthSmileRight,
    MouthFrownLeft,
    MouthFrownRight,
    MouthDimpleLeft,
    MouthDimpleRight,
    MouthStretchLeft,
    MouthStretchRight,
    MouthRollLower,
    MouthRollUpper,
    MouthShrugLower,
    MouthShrugUpper,
    MouthPressLeft,
    MouthPressRight,
    MouthLowerDownLeft,
    MouthLowerDownRight,
    MouthUpperUpLeft,
    MouthUpperUpRight,
    BrowDownLeft,
    BrowDownRight,
    BrowInnerUp,
    BrowOuterUpLeft,
    BrowOuterUpRight,
    CheekPuff,
    CheekSquintLeft,
    CheekSquintRight,
    NoseSneerLeft,
    NoseSneerRight,
    TongueOut,
    HeadYaw,
    HeadPitch,
    HeadRoll,
    LeftEyeYaw,
    LeftEyePitch,
    LeftEyeRoll,
    RightEyeYaw,
    RightEyePitch,
    RightEyeRoll,
}


pub fn map_range_clamped(val: f32, in_min: f32, in_max: f32, out_min: f32, out_max: f32) -> f32 {
    (val - in_min) / (in_max - in_min) * (out_max - out_min) + out_min
}

pub fn map_blend_shape(value: &ReadBlendShape, face_data_object: Gd<FaceDataObject>) -> f32 {
    let fdo = face_data_object.bind();
    match value {
        ReadBlendShape::EyeBlinkLeft => fdo.get_eye_blink_left(),
        ReadBlendShape::EyeLookDownLeft => fdo.get_eye_look_down_left(),
        ReadBlendShape::EyeLookInLeft => fdo.get_eye_look_in_left(),
        ReadBlendShape::EyeLookOutLeft => fdo.get_eye_look_out_left(),
        ReadBlendShape::EyeLookUpLeft => fdo.get_eye_look_up_left(),
        ReadBlendShape::EyeSquintLeft => fdo.get_eye_squint_left(),
        ReadBlendShape::EyeWideLeft => fdo.get_eye_wide_left(),
        ReadBlendShape::EyeBlinkRight => fdo.get_eye_blink_right(),
        ReadBlendShape::EyeLookDownRight => fdo.get_eye_look_down_right(),
        ReadBlendShape::EyeLookInRight => fdo.get_eye_look_in_right(),
        ReadBlendShape::EyeLookOutRight => fdo.get_eye_look_out_right(),
        ReadBlendShape::EyeLookUpRight => fdo.get_eye_look_up_right(),
        ReadBlendShape::EyeSquintRight => fdo.get_eye_squint_right(),
        ReadBlendShape::EyeWideRight => fdo.get_eye_wide_right(),
        ReadBlendShape::JawForward => fdo.get_jaw_forward(),
        ReadBlendShape::JawLeft => fdo.get_jaw_left(),
        ReadBlendShape::JawRight => fdo.get_jaw_right(),
        ReadBlendShape::JawOpen => fdo.get_jaw_open(),
        ReadBlendShape::MouthClose => fdo.get_mouth_close(),
        ReadBlendShape::MouthFunnel => fdo.get_mouth_funnel(),
        ReadBlendShape::MouthPucker => fdo.get_mouth_pucker(),
        ReadBlendShape::MouthLeft => fdo.get_mouth_left(),
        ReadBlendShape::MouthRight => fdo.get_mouth_right(),
        ReadBlendShape::MouthSmileLeft => fdo.get_mouth_smile_left(),
        ReadBlendShape::MouthSmileRight => fdo.get_mouth_smile_right(),
        ReadBlendShape::MouthFrownLeft => fdo.get_mouth_frown_left(),
        ReadBlendShape::MouthFrownRight => fdo.get_mouth_frown_right(),
        ReadBlendShape::MouthDimpleLeft => fdo.get_mouth_dimple_left(),
        ReadBlendShape::MouthDimpleRight => fdo.get_mouth_dimple_right(),
        ReadBlendShape::MouthStretchLeft => fdo.get_mouth_stretch_left(),
        ReadBlendShape::MouthStretchRight => fdo.get_mouth_stretch_right(),
        ReadBlendShape::MouthRollLower => fdo.get_mouth_roll_lower(),
        ReadBlendShape::MouthRollUpper => fdo.get_mouth_roll_upper(),
        ReadBlendShape::MouthShrugLower => fdo.get_mouth_shrug_lower(),
        ReadBlendShape::MouthShrugUpper => fdo.get_mouth_shrug_upper(),
        ReadBlendShape::MouthPressLeft => fdo.get_mouth_press_left(),
        ReadBlendShape::MouthPressRight => fdo.get_mouth_press_right(),
        ReadBlendShape::MouthLowerDownLeft => fdo.get_mouth_lower_down_left(),
        ReadBlendShape::MouthLowerDownRight => fdo.get_mouth_lower_down_right(),
        ReadBlendShape::MouthUpperUpLeft => fdo.get_mouth_upper_up_left(),
        ReadBlendShape::MouthUpperUpRight => fdo.get_mouth_upper_up_right(),
        ReadBlendShape::BrowDownLeft => fdo.get_brow_down_left(),
        ReadBlendShape::BrowDownRight => fdo.get_brow_down_right(),
        ReadBlendShape::BrowInnerUp => fdo.get_brow_inner_up(),
        ReadBlendShape::BrowOuterUpLeft => fdo.get_brow_outer_up_left(),
        ReadBlendShape::BrowOuterUpRight => fdo.get_brow_outer_up_right(),
        ReadBlendShape::CheekPuff => fdo.get_cheek_puff(),
        ReadBlendShape::CheekSquintLeft => fdo.get_cheek_squint_left(),
        ReadBlendShape::CheekSquintRight => fdo.get_cheek_squint_right(),
        ReadBlendShape::NoseSneerLeft => fdo.get_nose_sneer_left(),
        ReadBlendShape::NoseSneerRight => fdo.get_nose_sneer_right(),
        ReadBlendShape::TongueOut => fdo.get_tongue_out(),
        ReadBlendShape::HeadYaw => fdo.get_head_yaw(),
        ReadBlendShape::HeadPitch => fdo.get_head_pitch(),
        ReadBlendShape::HeadRoll => fdo.get_head_roll(),
        ReadBlendShape::LeftEyeYaw => fdo.get_left_eye_yaw(),
        ReadBlendShape::LeftEyePitch => fdo.get_left_eye_pitch(),
        ReadBlendShape::LeftEyeRoll => fdo.get_left_eye_roll(),
        ReadBlendShape::RightEyeYaw => fdo.get_right_eye_yaw(),
        ReadBlendShape::RightEyePitch => fdo.get_right_eye_pitch(),
        ReadBlendShape::RightEyeRoll => fdo.get_right_eye_roll(),
    }
}
