use godot::prelude::*;
use live_link_face::{FaceBlendShape, LiveLinkFace};

#[derive(GodotClass)]
#[class(base=RefCounted)]
pub struct FaceDataObject {
    #[var]
    eye_blink_left: f32,
    #[var]
    eye_look_down_left: f32,
    #[var]
    eye_look_in_left: f32,
    #[var]
    eye_look_out_left: f32,
    #[var]
    eye_look_up_left: f32,
    #[var]
    eye_squint_left: f32,
    #[var]
    eye_wide_left: f32,

    #[var]
    eye_blink_right: f32,
    #[var]
    eye_look_down_right: f32,
    #[var]
    eye_look_in_right: f32,
    #[var]
    eye_look_out_right: f32,
    #[var]
    eye_look_up_right: f32,
    #[var]
    eye_squint_right: f32,
    #[var]
    eye_wide_right: f32,

    #[var]
    jaw_forward: f32,
    #[var]
    jaw_left: f32,
    #[var]
    jaw_right: f32,
    #[var]
    jaw_open: f32,

    #[var]
    mouth_close: f32,
    #[var]
    mouth_funnel: f32,
    #[var]
    mouth_pucker: f32,
    #[var]
    mouth_left: f32,
    #[var]
    mouth_right: f32,
    #[var]
    mouth_smile_left: f32,
    #[var]
    mouth_smile_right: f32,
    #[var]
    mouth_frown_left: f32,
    #[var]
    mouth_frown_right: f32,
    #[var]
    mouth_dimple_left: f32,
    #[var]
    mouth_dimple_right: f32,
    #[var]
    mouth_stretch_left: f32,
    #[var]
    mouth_stretch_right: f32,
    #[var]
    mouth_roll_lower: f32,
    #[var]
    mouth_roll_upper: f32,
    #[var]
    mouth_shrug_lower: f32,
    #[var]
    mouth_shrug_upper: f32,
    #[var]
    mouth_press_left: f32,
    #[var]
    mouth_press_right: f32,
    #[var]
    mouth_lower_down_left: f32,
    #[var]
    mouth_lower_down_right: f32,
    #[var]
    mouth_upper_up_left: f32,
    #[var]
    mouth_upper_up_right: f32,

    #[var]
    brow_down_left: f32,
    #[var]
    brow_down_right: f32,
    #[var]
    brow_inner_up: f32,
    #[var]
    brow_outer_up_left: f32,
    #[var]
    brow_outer_up_right: f32,

    #[var]
    cheek_puff: f32,
    #[var]
    cheek_squint_left: f32,
    #[var]
    cheek_squint_right: f32,

    #[var]
    nose_sneer_left: f32,
    #[var]
    nose_sneer_right: f32,

    #[var]
    tongue_out: f32,

    #[var]
    head_yaw: f32,
    #[var]
    head_pitch: f32,
    #[var]
    head_roll: f32,

    #[var]
    left_eye_yaw: f32,
    #[var]
    left_eye_pitch: f32,
    #[var]
    left_eye_roll: f32,

    #[var]
    right_eye_yaw: f32,
    #[var]
    right_eye_pitch: f32,
    #[var]
    right_eye_roll: f32,

    base: Base<RefCounted>,
}

pub fn read_face_data(face_data_object: &mut Gd<FaceDataObject>, face_data: &LiveLinkFace) {
    let mut face_data_object = face_data_object.bind_mut();

    face_data_object.eye_blink_left = face_data.get_blendshape(FaceBlendShape::EyeBlinkLeft);

    face_data_object.eye_look_down_left = face_data.get_blendshape(FaceBlendShape::EyeLookDownLeft);

    face_data_object.eye_look_in_left = face_data.get_blendshape(FaceBlendShape::EyeLookInLeft);

    face_data_object.eye_look_out_left = face_data.get_blendshape(FaceBlendShape::EyeLookOutLeft);

    face_data_object.eye_look_up_left = face_data.get_blendshape(FaceBlendShape::EyeLookUpLeft);

    face_data_object.eye_squint_left = face_data.get_blendshape(FaceBlendShape::EyeSquintLeft);

    face_data_object.eye_wide_left = face_data.get_blendshape(FaceBlendShape::EyeWideLeft);

    face_data_object.eye_blink_right = face_data.get_blendshape(FaceBlendShape::EyeBlinkRight);

    face_data_object.eye_look_down_right =
        face_data.get_blendshape(FaceBlendShape::EyeLookDownRight);

    face_data_object.eye_look_in_right = face_data.get_blendshape(FaceBlendShape::EyeLookInRight);

    face_data_object.eye_look_out_right = face_data.get_blendshape(FaceBlendShape::EyeLookOutRight);

    face_data_object.eye_look_up_right = face_data.get_blendshape(FaceBlendShape::EyeLookUpRight);

    face_data_object.eye_squint_right = face_data.get_blendshape(FaceBlendShape::EyeSquintRight);

    face_data_object.eye_wide_right = face_data.get_blendshape(FaceBlendShape::EyeWideRight);

    face_data_object.jaw_forward = face_data.get_blendshape(FaceBlendShape::JawForward);

    face_data_object.jaw_left = face_data.get_blendshape(FaceBlendShape::JawLeft);

    face_data_object.jaw_right = face_data.get_blendshape(FaceBlendShape::JawRight);

    face_data_object.jaw_open = face_data.get_blendshape(FaceBlendShape::JawOpen);

    face_data_object.mouth_close = face_data.get_blendshape(FaceBlendShape::MouthClose);

    face_data_object.mouth_funnel = face_data.get_blendshape(FaceBlendShape::MouthFunnel);

    face_data_object.mouth_pucker = face_data.get_blendshape(FaceBlendShape::MouthPucker);

    face_data_object.mouth_left = face_data.get_blendshape(FaceBlendShape::MouthLeft);

    face_data_object.mouth_right = face_data.get_blendshape(FaceBlendShape::MouthRight);

    face_data_object.mouth_smile_left = face_data.get_blendshape(FaceBlendShape::MouthSmileLeft);

    face_data_object.mouth_smile_right = face_data.get_blendshape(FaceBlendShape::MouthSmileRight);

    face_data_object.mouth_frown_left = face_data.get_blendshape(FaceBlendShape::MouthFrownLeft);

    face_data_object.mouth_frown_right = face_data.get_blendshape(FaceBlendShape::MouthFrownRight);

    face_data_object.mouth_dimple_left = face_data.get_blendshape(FaceBlendShape::MouthDimpleLeft);

    face_data_object.mouth_dimple_right =
        face_data.get_blendshape(FaceBlendShape::MouthDimpleRight);

    face_data_object.mouth_stretch_left =
        face_data.get_blendshape(FaceBlendShape::MouthStretchLeft);

    face_data_object.mouth_stretch_right =
        face_data.get_blendshape(FaceBlendShape::MouthStretchRight);

    face_data_object.mouth_roll_lower = face_data.get_blendshape(FaceBlendShape::MouthRollLower);

    face_data_object.mouth_roll_upper = face_data.get_blendshape(FaceBlendShape::MouthRollUpper);

    face_data_object.mouth_shrug_lower = face_data.get_blendshape(FaceBlendShape::MouthShrugLower);

    face_data_object.mouth_shrug_upper = face_data.get_blendshape(FaceBlendShape::MouthShrugUpper);

    face_data_object.mouth_press_left = face_data.get_blendshape(FaceBlendShape::MouthPressLeft);

    face_data_object.mouth_press_right = face_data.get_blendshape(FaceBlendShape::MouthPressRight);

    face_data_object.mouth_lower_down_left =
        face_data.get_blendshape(FaceBlendShape::MouthLowerDownLeft);

    face_data_object.mouth_lower_down_right =
        face_data.get_blendshape(FaceBlendShape::MouthLowerDownRight);

    face_data_object.mouth_upper_up_left =
        face_data.get_blendshape(FaceBlendShape::MouthUpperUpLeft);

    face_data_object.mouth_upper_up_right =
        face_data.get_blendshape(FaceBlendShape::MouthUpperUpRight);

    face_data_object.brow_down_left = face_data.get_blendshape(FaceBlendShape::BrowDownLeft);

    face_data_object.brow_down_right = face_data.get_blendshape(FaceBlendShape::BrowDownRight);

    face_data_object.brow_inner_up = face_data.get_blendshape(FaceBlendShape::BrowInnerUp);

    face_data_object.brow_outer_up_left = face_data.get_blendshape(FaceBlendShape::BrowOuterUpLeft);

    face_data_object.brow_outer_up_right =
        face_data.get_blendshape(FaceBlendShape::BrowOuterUpRight);

    face_data_object.cheek_puff = face_data.get_blendshape(FaceBlendShape::CheekPuff);

    face_data_object.cheek_squint_left = face_data.get_blendshape(FaceBlendShape::CheekSquintLeft);

    face_data_object.cheek_squint_right =
        face_data.get_blendshape(FaceBlendShape::CheekSquintRight);

    face_data_object.nose_sneer_left = face_data.get_blendshape(FaceBlendShape::NoseSneerLeft);

    face_data_object.nose_sneer_right = face_data.get_blendshape(FaceBlendShape::NoseSneerRight);

    face_data_object.tongue_out = face_data.get_blendshape(FaceBlendShape::TongueOut);

    face_data_object.head_yaw = face_data.get_blendshape(FaceBlendShape::HeadYaw);

    face_data_object.head_pitch = face_data.get_blendshape(FaceBlendShape::HeadPitch);

    face_data_object.head_roll = face_data.get_blendshape(FaceBlendShape::HeadRoll);

    face_data_object.left_eye_yaw = face_data.get_blendshape(FaceBlendShape::LeftEyeYaw);

    face_data_object.left_eye_pitch = face_data.get_blendshape(FaceBlendShape::LeftEyePitch);

    face_data_object.left_eye_roll = face_data.get_blendshape(FaceBlendShape::LeftEyeRoll);

    face_data_object.right_eye_yaw = face_data.get_blendshape(FaceBlendShape::RightEyeYaw);

    face_data_object.right_eye_pitch = face_data.get_blendshape(FaceBlendShape::RightEyePitch);

    face_data_object.right_eye_roll = face_data.get_blendshape(FaceBlendShape::RightEyeRoll);
}

#[godot_api]
impl IRefCounted for FaceDataObject {
    fn init(base: Base<RefCounted>) -> Self {
        Self {
            eye_blink_left: 0.0,
            eye_look_down_left: 0.0,
            eye_look_in_left: 0.0,
            eye_look_out_left: 0.0,
            eye_look_up_left: 0.0,
            eye_squint_left: 0.0,
            eye_wide_left: 0.0,

            eye_blink_right: 0.0,
            eye_look_down_right: 0.0,
            eye_look_in_right: 0.0,
            eye_look_out_right: 0.0,
            eye_look_up_right: 0.0,
            eye_squint_right: 0.0,
            eye_wide_right: 0.0,

            jaw_forward: 0.0,
            jaw_left: 0.0,
            jaw_right: 0.0,
            jaw_open: 0.0,

            mouth_close: 0.0,
            mouth_funnel: 0.0,
            mouth_pucker: 0.0,
            mouth_left: 0.0,
            mouth_right: 0.0,
            mouth_smile_left: 0.0,
            mouth_smile_right: 0.0,
            mouth_frown_left: 0.0,
            mouth_frown_right: 0.0,
            mouth_dimple_left: 0.0,
            mouth_dimple_right: 0.0,
            mouth_stretch_left: 0.0,
            mouth_stretch_right: 0.0,
            mouth_roll_lower: 0.0,
            mouth_roll_upper: 0.0,
            mouth_shrug_lower: 0.0,
            mouth_shrug_upper: 0.0,
            mouth_press_left: 0.0,
            mouth_press_right: 0.0,
            mouth_lower_down_left: 0.0,
            mouth_lower_down_right: 0.0,
            mouth_upper_up_left: 0.0,
            mouth_upper_up_right: 0.0,

            brow_down_left: 0.0,
            brow_down_right: 0.0,
            brow_inner_up: 0.0,
            brow_outer_up_left: 0.0,
            brow_outer_up_right: 0.0,

            cheek_puff: 0.0,
            cheek_squint_left: 0.0,
            cheek_squint_right: 0.0,

            nose_sneer_left: 0.0,
            nose_sneer_right: 0.0,

            tongue_out: 0.0,

            head_yaw: 0.0,
            head_pitch: 0.0,
            head_roll: 0.0,

            left_eye_yaw: 0.0,
            left_eye_pitch: 0.0,
            left_eye_roll: 0.0,

            right_eye_yaw: 0.0,
            right_eye_pitch: 0.0,
            right_eye_roll: 0.0,

            base,
        }
    }
}
